package com.marvinelsen.chinese.phonetics

import com.marvinelsen.chinese.phonetics.internal.tone.formatting.PinyinNumberToneFormatter
import com.marvinelsen.chinese.phonetics.internal.tone.formatting.PinyinToneFormatter
import com.marvinelsen.chinese.phonetics.internal.tone.formatting.ZhuyinToneFormatter
import com.marvinelsen.chinese.phonetics.internal.tone.parsing.DigitToneParser
import com.marvinelsen.chinese.phonetics.internal.tone.parsing.IntToneParser
import com.marvinelsen.chinese.phonetics.internal.tone.parsing.PinyinToneParser
import com.marvinelsen.chinese.phonetics.internal.tone.parsing.ZhuyinToneParser

enum class Tone {
    FIRST, SECOND, THIRD, FOURTH, FIFTH;

    companion object {
        fun fromInt(toneNumber: Int) = IntToneParser.parse(toneNumber)
        fun fromDigit(digit: Char) = DigitToneParser.parse(digit)
        fun fromPinyinTone(pinyinTone: Char) = PinyinToneParser.parse(pinyinTone)
        fun fromZhuyinTone(zhuyinTone: Char) = ZhuyinToneParser.parse(zhuyinTone)

        fun fromIntOrNull(toneNumber: Int) = IntToneParser.parseOrNull(toneNumber)
        fun fromDigitOrNull(digit: Char) = DigitToneParser.parseOrNull(digit)
        fun fromPinyinToneOrNull(pinyinTone: Char) = PinyinToneParser.parseOrNull(pinyinTone)
        fun fromZhuyinToneOrNull(zhuyinTone: Char) = ZhuyinToneParser.parseOrNull(zhuyinTone)
    }
}

@Suppress("MagicNumber")
fun Tone.toInt() = when (this) {
    Tone.FIRST -> 1
    Tone.SECOND -> 2
    Tone.THIRD -> 3
    Tone.FOURTH -> 4
    Tone.FIFTH -> 5
}

fun Tone.toPinyinNumber() = PinyinNumberToneFormatter.format(this)
fun Tone.toPinyinTone() = PinyinToneFormatter.format(this)
fun Tone.toZhuyinTone() = ZhuyinToneFormatter.format(this)
