package com.marvinelsen.chinese.phonetics.internal.syllable.formatting

import com.marvinelsen.chinese.phonetics.Tone
import com.marvinelsen.chinese.phonetics.internal.syllable.Syllable
import com.marvinelsen.chinese.phonetics.toPinyinTone

internal data object PinyinMarkSyllableFormatter : SyllableFormatter {
    private fun findVowelIndexForToneMark(pinyin: String): Int {
        val characterToIndex = pinyin.withIndex().associate { it.value to it.index }
        return when {
            'a' in characterToIndex -> characterToIndex['a']!!
            'o' in characterToIndex -> characterToIndex['o']!!
            'e' in characterToIndex -> characterToIndex['e']!!
            'i' in characterToIndex ->
                if (pinyin.elementAtOrNull(characterToIndex['i']!! + 1) == 'u') {
                    characterToIndex['u']!!
                } else {
                    characterToIndex['i']!!
                }

            'u' in characterToIndex -> characterToIndex['u']!!
            'ü' in characterToIndex -> characterToIndex['ü']!!
            else -> error("No vowel found in Pinyin syllable '$pinyin'")
        }
    }

    override fun format(syllable: Syllable): String {
        if (syllable.tone == Tone.FIFTH) return syllable.basePinyin

        val vowelIndex = findVowelIndexForToneMark(syllable.basePinyin)
        val toneMark = syllable.tone.toPinyinTone()

        return buildString {
            append(syllable.basePinyin)
            insert(vowelIndex + 1, toneMark)
        }
    }
}
