package com.marvinelsen.chinese.phonetics.internal.syllable.parsing

import com.marvinelsen.chinese.phonetics.Tone
import com.marvinelsen.chinese.phonetics.internal.syllable.Syllable
import com.marvinelsen.chinese.phonetics.internal.syllable.TranscriptionDataRepository

internal data object PinyinNumberSyllableParser : SyllableParser {
    @Suppress("ReturnCount", "MagicNumber")
    override fun parseOrNull(input: String): Syllable? {
        if (input.isBlank() || !input.last().isDigit()) return null

        val toneNumber = input.last()
        val tone = Tone.fromDigitOrNull(toneNumber) ?: return null

        val pinyin = input.substring(0, input.lastIndex)
        val normalizedPinyin = TranscriptionDataRepository.normalize(pinyin)

        if (!TranscriptionDataRepository.isValidPinyin(normalizedPinyin)) return null

        return Syllable(
            basePinyin = normalizedPinyin,
            tone = tone
        )
    }
}
