package com.marvinelsen.chinese.phonetics.internal.syllable.parsing

import com.marvinelsen.chinese.phonetics.Tone
import com.marvinelsen.chinese.phonetics.internal.syllable.Syllable
import com.marvinelsen.chinese.phonetics.internal.syllable.TranscriptionDataRepository

internal data object ZhuyinSyllableParser : SyllableParser {
    private val zhuyinToneMarkRegex = """[ˊˇˋ˙¯]""".toRegex()

    override fun parseOrNull(input: String): Syllable? {
        val zhuyinWithoutToneMark = input.replace(zhuyinToneMarkRegex, "")

        if (!TranscriptionDataRepository.isValidZhuyin(zhuyinWithoutToneMark)) return null

        return Syllable(
            basePinyin = TranscriptionDataRepository.zhuyinToPinyin[zhuyinWithoutToneMark]!!,
            tone = Tone.fromZhuyinToneOrNull(input.last()) ?: Tone.fromZhuyinToneOrNull(input.first()) ?: Tone.FIRST
        )
    }
}
